var SmartBillCloud = (function ($j) {
    var $invoiceSeries,
        $proformaSeries;
    function _init() {
        displayMessages();
        $invoiceSeries = $j('#SMARTBILL_INVOICE_SERIES').parent().parent();
        $proformaSeries = $j('#SMARTBILL_ESTIMATE_SERIES').parent().parent();
        _initAjaxCalls();
        _addOrderColumn();
        _initDisplayTransport();
        _initIssueWithDueDate();
        _initIssueWithDeliveryDays();
        _initIssueWithDelegateData();
        _initSyncStock();
        _initMoveInvoicePannelBackOffice();
        _initChangeOrderStatus();
        _initAutomaticallyIssueDocument();
        _initSendMailDocument();
        _initManualStockSync();
        _customMeasuringUnit();
        _initHideDelegateFields();
        add_copy_to_clipboard();
    }

    function _initHideDelegateFields() {
        if ($j("#SMARTBILL_ADD_DELEGATE_DATA_off").length && $j("#SMARTBILL_ADD_DELEGATE_DATA_off").is(':checked')) {
            $j('.smartbill-show-if-add-delegate-data-on').closest('.form-group').hide();
        }

        $j('#SMARTBILL_ADD_DELEGATE_DATA_off').on('change', function (e) {
            $j('.smartbill-show-if-add-delegate-data-on').closest('.form-group').hide();
        })

        $j('#SMARTBILL_ADD_DELEGATE_DATA_on').on('change', function (e) {
            $j('.smartbill-show-if-add-delegate-data-on').closest('.form-group').show();
        });
    }

    function _initManualStockSync() {
        /**
        * Manually sync stock.
        *
        * Send ajax call for retrieving stocks from smartbill.
        * 
        * @since      3.1.3
        * 
        * @return {void}
        */
        $("#smartbill-manually-sync-stock").click(function (e) {
            e.preventDefault();
            var warehouse = $("#SMARTBILL_USED_STOCK").find(":selected").val();
            var info_message = document.createElement("div");
            if ('' == warehouse || 'fara-gestiune' == warehouse) {
                Toastify({
                    text: "Este necesara selectarea gestiunii din care vor fi preluate stocurile.", duration: -1, newWindow: false, close: true, gravity: "top", position: 'center', backgroundColor: "#EF4136", stopOnFocus: true,
                }).showToast();
            } else {
                info_message.innerHTML = "Stocurile produselor din Prestashop <strong>vor fi actualizate</strong> cu stocurile produselor din <strong>gestiunea " + warehouse + "</strong>."
                swal.fire({
                    title: 'Atentie!',
                    html: info_message,
                    icon: 'warning',
                    showCancelButton: true,
                    buttonsStyling: true,
                    reverseButtons: true,
                    confirmButtonText: 'Actualizeaza stocuri',
                    cancelButtonText: 'Renunta'
                }).then(function (result) {
                    if (result.isDismissed) {
                        return false;
                    };

                    info_message = document.createElement("div");
                    info_message.innerHTML = "Va rugam asteptati finalizarea preluarii...";

                    swal.fire({
                        title: "Se incarca!",
                        html: info_message,
                        icon: 'info',
                        buttonsStyling: true,
                        reverseButtons: true,
                        showConfirmButton: false
                    });

                    $.ajax({
                        url: ajax_url + "&warehouse=" + warehouse + "&action=syncStock",
                        success: function (response) {
                            try {
                                response = JSON.parse(response);
                                info_message = document.createElement("div")
                                info_message.innerHTML = response.data;
                                if (undefined != response.data) {
                                    swal.close();
                                    swal.fire({
                                        title: "",
                                        html: info_message,
                                        icon: response.icon,
                                        showCancelButton: true,
                                        buttonsStyling: true,
                                        reverseButtons: true,
                                        confirmButtonText: 'Descarca istoric',
                                        cancelButtonText: 'Am inteles'
                                    }).then(function (result) {
                                        if (result.isDismissed) {
                                            return false;
                                        };
                                        $.ajax({
                                            url: ajax_url + "&action=downloadManualStockHistory",
                                            xhr: function () {
                                                var xhr = new XMLHttpRequest();
                                                xhr.onreadystatechange = function () {
                                                    if (xhr.readyState == 2) {
                                                        if (xhr.status == 200) {
                                                            xhr.responseType = "blob";
                                                        } else {
                                                            xhr.responseType = "text";
                                                        }
                                                    }
                                                };
                                                return xhr;
                                            },
                                            success: function (data) {
                                                var blob = new Blob([data], { type: "application/octetstream" });

                                                //Check the Browser type and download the File.
                                                var isIE = false || !!document.documentMode;
                                                if (isIE) {
                                                    window.navigator.msSaveBlob(blob, "Istoric sincronizare manuala SmartBill.zip");
                                                } else {
                                                    var url = window.URL || window.webkitURL;
                                                    link = url.createObjectURL(blob);
                                                    var a = $("<a />");
                                                    a.attr("download", "Istoric sincronizare manuala SmartBill.zip");
                                                    a.attr("href", link);
                                                    $("body").append(a);
                                                    a[0].click();
                                                    $("body").remove(a);
                                                }
                                            },
                                            error: function (request, status, error) {
                                                errorMessages.push(error);
                                                saveMessages();
                                                location.reload();
                                            }
                                        });
                                    });
                                } else {
                                    throw response.error;
                                }
                            } catch (error) {
                                errorMessages.push(error);
                                saveMessages();
                                location.reload();
                            }
                        },
                        error: function (request, status, error) {
                            errorMessages.push(error);
                            saveMessages();
                            location.reload();
                        }
                    });

                });
            }


        });
    }

    function add_copy_to_clipboard() {
        $("#url_container_stocks input").click(function () {
            navigator.clipboard.writeText($("#url_container_stocks input").val());
            Toastify({
                text: "URL-ul a fost copiat cu succes!", duration: 3000, newWindow: false, close: true, gravity: "top", position: 'center', backgroundColor: "#00A14B", stopOnFocus: true,
            }).showToast();
        });
    }

    function _customMeasuringUnit() {
        //display error
        if ($('#SMARTBILL_UM').find('option:selected').val() == 'no_value' && !$('.module_error ').length) {
            Toastify({
                text: "Este necesara selectarea unei unitati de masura.", duration: -1, newWindow: false, close: true, gravity: "top", position: 'center', backgroundColor: "#EF4136", stopOnFocus: true,
            }).showToast();
        }

    };

    function _initSendMailDocument() {
        if ($j('#SMARTBILL_SEND_MAIL_WITH_DOCUMENT_off').prop('checked')) {
            $('#SMARTBILL_SEND_MAIL_CC').closest('.form-group').hide();
            $('#SMARTBILL_SEND_MAIL_BCC').closest('.form-group').hide();
        } else {
            $('#SMARTBILL_SEND_MAIL_CC').closest('.form-group').show();
            $('#SMARTBILL_SEND_MAIL_BCC').closest('.form-group').show();
        }
        $j('#SMARTBILL_SEND_MAIL_WITH_DOCUMENT_off').on('change', function (e) {
            $('#SMARTBILL_SEND_MAIL_CC').closest('.form-group').hide();
            $('#SMARTBILL_SEND_MAIL_BCC').closest('.form-group').hide();
        });
        $j('#SMARTBILL_SEND_MAIL_WITH_DOCUMENT_on').on('change', function (e) {
            $('#SMARTBILL_SEND_MAIL_CC').closest('.form-group').show();
            $('#SMARTBILL_SEND_MAIL_BCC').closest('.form-group').show();
        });
    }

    function _initChangeOrderStatus() {
        // 1.7.6
        $j('#submit_state').click(function (e) {
            if (!e.detail || e.detail == 1) {
                _changeOneOrderStatus('#id_order_state', e, '#submit_state');
            }
        });
        $j('button[name="submitUpdateOrderStatus"]').click(
            {
                inputs_el: "#table-order input[name='orderBox[]']:checked",
                old: true,
                order_status_el: '#id_order_state'
            }, _changeBulkOrderStatus
        );


        //1.7.7
        $j('.js-dropdown-item.dropdown-item').click(function (e) {
            const $button = $(e.currentTarget);
            const $parent = $button.closest('.js-choice-options');
            const url = $parent.data('url');

            if (smartbill.automatically_issue_document == '1') {
                var order_status = $j(this)[0].dataset.value;
                if (order_status == smartbill.order_status) {
                    e.preventDefault();
                    e.stopImmediatePropagation();
                    var reisue = $j(this).closest('tr').find('.smartbill_prestashop_invoice').find('a');
                    if (reisue.length === 0) {
                        var order_id = $j(this).closest('tr').find('.js-bulk-action-checkbox').val();
                        var attention_text = document.createElement("div");
                        attention_text.innerHTML = "Comanda #" + order_id + " va fi emisa in SmartBill.";
                        swal.fire({
                            title: "Modulul SmartBill a detectat modificarea statusului comenzii.",
                            html: attention_text,
                            icon: 'warning',
                            showCancelButton: true,
                            buttonsStyling: true,
                            reverseButtons: true,
                            confirmButtonText: 'Emite document',
                            cancelButtonText: 'Mai tarziu'
                        }).then(async function (result) {
                            if (result.isDismissed) {
                                submitForm(url, $button);
                                return false;
                            }
                            var payload = ajax_url + order_id + "&action=exportDocument";
                            await call_issue_doc(payload).then(function () {
                                submitForm(url, $button);
                            });
                        });
                    } else {
                        var attention_text = document.createElement("div");
                        attention_text.innerHTML = "Te informam ca nu se va emite automat o alta comanda in SmartBill.";
                        swal.fire({
                            title: "Comanda a fost deja facturata in SmartBill. ",
                            html: attention_text,
                            icon: 'warning',
                            buttonsStyling: true,
                            reverseButtons: true,
                            confirmButtonText: 'Continua'
                        }).then(function (result) {
                            submitForm(url, $button);
                        });
                    }
                }
            }
            // submitForm(url, $button);
        });

        submitForm = function (url, $button) {
            const selectedStatusId = $button.data('value');
            const $form = $('<form>', {
                action: url,
                method: 'POST',
            }).append(
                $('<input>', {
                    name: 'value',
                    value: selectedStatusId,
                    type: 'hidden',
                }));

            $form.appendTo('body');
            $form.submit();
        };


        $j('#update_order_status_new_order_status_id').parent().find('button').click(function (e) {
            if (!e.detail || e.detail == 1) {
                _changeOneOrderStatus('#update_order_status_new_order_status_id', e, '');
            }
        });

        $j('#update_order_status_action_btn').click(function (e) {
            if (!e.detail || e.detail == 1) {
                _changeOneOrderStatus('#update_order_status_action_input', e, '#update_order_status_action_btn');
            }
        });

        $j('#changeOrdersStatusModal form').submit(
            {
                inputs_el: "#order_grid_table input[name='order_orders_bulk[]']:checked",
                old: false,
                order_status_el: '#change_orders_status_new_order_status_id'
            }, _changeBulkOrderStatus
        );
    }

    function _changeOneOrderStatus(selectOrdStatus, e, form) {
        if (smartbill.automatically_issue_document == '1') {
            if ($j(selectOrdStatus).val() == smartbill.order_status) {
                if ($('.smartbill-container a.smrt-ajax.reissue').length === 0) {
                    e.preventDefault();
                    var attention_text = document.createElement("div");
                    attention_text.innerHTML = "Comanda #" + smartbill.post_id + " va fi emisa in SmartBill.";
                    swal.fire({
                        title: "Modulul SmartBill a detectat modificarea statusului comenzii.",
                        html: attention_text,
                        icon: 'warning',
                        showCancelButton: true,
                        buttonsStyling: true,
                        reverseButtons: true,
                        confirmButtonText: 'Emite document',
                        cancelButtonText: 'Mai tarziu'
                    }).then(async function (result) {
                        if (result.isDismissed) {
                            $j(form).off('click').trigger('click');
                            return false;
                        }
                        var api_call = $('.smrt-ajax').data('href');
                        await call_issue_doc(api_call).then(function () {
                            if (form != '') {
                                $j(form).off('click').trigger('click');
                            } else {
                                $(e.currentTarget.form).submit();
                            }
                        });
                    });
                } else {
                    e.preventDefault();
                    var attention_text = document.createElement("div");
                    attention_text.innerHTML = "Te informam ca nu se va emite automat o alta comanda in SmartBill.";
                    swal.fire({
                        title: "Comanda a fost deja facturata in SmartBill. ",
                        html: attention_text,
                        icon: 'warning',
                        buttonsStyling: true,
                        reverseButtons: true,
                        confirmButtonText: 'Continua'
                    }).then(function (result) {
                        if (form != '') {
                            $j(form).off('click').trigger('click');
                        } else {
                            $(e.currentTarget.form).submit();
                        }
                    });
                }
            }
        }
    }
    function _changeBulkOrderStatus(e) {
        if (smartbill.automatically_issue_document == '1') {
            if (!e.detail || e.detail == 1) {
                var pending_orders = [];
                var orders_with_invoices = [];

                var inputs = $j(e.data.inputs_el);
                inputs.each(function (k, elem) {
                    var has_document = $j(elem).closest('tr').find('.smartbill_prestashop_invoice').find('a');
                    if (has_document.length) {
                        orders_with_invoices.push(elem);
                    } else {
                        pending_orders.push(elem);
                    }
                });
                pending_orders.sort((a, b) => a.value - b.value);
                orders_with_invoices.sort((a, b) => a.value - b.value);

                if ($j(e.data.order_status_el).val() == smartbill.order_status) {
                    e.preventDefault();
                    if (pending_orders.length) {
                        if (orders_with_invoices.length) {
                            var attention_text = document.createElement("div");
                            var swal_title = "Modulul SmartBill a detectat modificarea statusului comenzilor.";
                            var orders = "";
                            pending_orders.forEach((item, index) => {
                                if (index != pending_orders.length - 1) {
                                    orders += "#" + item.value + ', ';
                                } else {
                                    orders += "#" + item.value;
                                }
                            });
                            attention_text.innerHTML = "Comenzile " + orders + " vor fi emise in SmartBill.";
                            if (pending_orders.length == 1) {
                                swal_title = "Modulul SmartBill a detectat modificarea statusului comenzii.";
                                attention_text.innerHTML = "Comanda " + orders + " va fi emisa in SmartBill.";
                            }
                            swal.fire({
                                title: swal_title,
                                html: attention_text,
                                icon: 'warning',
                                showCancelButton: true,
                                buttonsStyling: true,
                                reverseButtons: true,
                                confirmButtonText: 'Emite in SmartBill',
                                cancelButtonText: 'Mai tarziu'
                                
                            }).then(function (result) {
                                if (result.isDismissed) {
                                    if (e.data.old) {
                                        $j('button[name="submitUpdateOrderStatus"]').off('click').trigger('click');
                                    } else {
                                        $j('#changeOrdersStatusModal form').off('submit').submit();
                                    }

                                    return false;
                                }

                                var info_text = document.createElement("div");
                                info_text.innerHTML = "Va rugam asteptati raspunsul serverului...";
                                swal.fire({
                                    title: 'Se incarca!',
                                    html: info_text,
                                    icon: 'info',
                                    buttonsStyling: true,
                                    reverseButtons: true,
                                    showConfirmButton: false
                                });

                                const start = async () => {
                                    await asyncForEach(pending_orders, async (item) => {
                                        var order_id = parseInt($(item).val());
                                        var payload = ajax_url + order_id + "&action=exportDocument";
                                        var result = await call_issue_doc(payload, false);
                                        result = JSON.parse(result);
                                        if (result.hasOwnProperty('headers')) {
                                            if (parseInt(result.headers['x-ratelimit-remaining']) <= 4) {
                                                var server_date = Math.floor(new Date(parseInt(result.headers['x-ratelimit-reset']) * 1000).getTime() / 1000.0);
                                                var curentr_date = Math.floor(new Date().getTime() / 1000.0);
                                                var time_to_wait = Math.abs(curentr_date - server_date) * (4 - parseInt(result.headers['x-ratelimit-remaining']));
                                                await waitFor(time_to_wait);
                                            }
                                        }
                                    });
                                }
                                start().then(function () {
                                    swal.close();
                                    if (e.data.old) {
                                        $j('button[name="submitUpdateOrderStatus"]').off('click').trigger('click');
                                    } else {
                                        $j('#changeOrdersStatusModal form').off('submit').submit();
                                    }
                                });
                            });
                        } else {
                            var attention_text = document.createElement("div");
                            var orders = "";
                            var swal_title = "Modulul SmartBill a detectat modificarea statusului comenzilor.";
                            pending_orders.forEach((item, index) => {
                                if (index != pending_orders.length - 1) {
                                    orders += "#" + item.value + ', ';
                                } else {
                                    orders += "#" + item.value;
                                }
                            });
                            attention_text.innerHTML = "Comenzile " + orders + " vor fi emise in SmartBill.";
                            if (pending_orders.length == 1) {
                                swal_title = "Modulul SmartBill a detectat modificarea statusului comenzii.";
                                attention_text.innerHTML = "Comanda " + orders + " va fi emisa in SmartBill.";
                            }
                            swal.fire({
                                title: swal_title,
                                html: attention_text,
                                icon: 'warning',
                                showCancelButton: true,
                                buttonsStyling: true,
                                reverseButtons: true,
                                confirmButtonText: 'Emite in SmartBill',
                                cancelButtonText: 'Mai tarziu'
                            }).then(function (result) {
                                if (result.isDismissed) {
                                    if (e.data.old) {
                                        $j('button[name="submitUpdateOrderStatus"]').off('click').trigger('click');
                                    } else {
                                        $j('#changeOrdersStatusModal form').off('submit').submit();
                                    }
                                    return false;
                                }

                                var info_text = document.createElement("div");
                                info_text.innerHTML = "Va rugam asteptati raspunsul serverului...";
                                swal.fire({
                                    title: 'Se incarca!',
                                    html: info_text,
                                    icon: 'info',
                                    buttonsStyling: true,
                                    reverseButtons: true,
                                    showConfirmButton: false
                                });

                                const start = async () => {
                                    await asyncForEach(pending_orders, async (item) => {
                                        var order_id = parseInt($(item).val());
                                        var payload = ajax_url + order_id + "&action=exportDocument";
                                        var result = await call_issue_doc(payload, false);
                                        result = JSON.parse(result);
                                        if (result.hasOwnProperty('headers')) {
                                            if (parseInt(result.headers['x-ratelimit-remaining']) <= 4) {
                                                var server_date = Math.floor(new Date(parseInt(result.headers['x-ratelimit-reset']) * 1000).getTime() / 1000.0);
                                                var curentr_date = Math.floor(new Date().getTime() / 1000.0);
                                                var time_to_wait = Math.abs(curentr_date - server_date) * (4 - parseInt(result.headers['x-ratelimit-remaining']));
                                                await waitFor(time_to_wait);
                                            }
                                        }
                                    });
                                }
                                start().then(function () {
                                    swal.close();
                                    if (e.data.old) {
                                        $j('button[name="submitUpdateOrderStatus"]').off('click').trigger('click');
                                    } else {
                                        $j('#changeOrdersStatusModal form').off('submit').submit();
                                    }
                                });
                            });
                        }
                    } else {
                        if (e.data.old) {
                            $j('button[name="submitUpdateOrderStatus"]').off('click').trigger('click');
                        } else {
                            $j('#changeOrdersStatusModal form').off('submit').submit();
                        }
                    }

                }
            }
        }
    }

    function _initAutomaticallyIssueDocument() {
        //document type changes are also inside this function
        $j('#SMARTBILL_DOCUMENT_TYPE').on('change', function (e) {
            switch ($j(this).val()) {
                // proforma
                case '1':
                    $invoiceSeries.hide();
                    $proformaSeries.show();
                    break;

                // invoice
                default:
                    $invoiceSeries.show();
                    $proformaSeries.hide();
                    break;
            }
        }).trigger('change');

        if ($j('#SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT_off').prop('checked')) {
            $('#SMARTBILL_ORDER_STATUS').parent().parent().hide();
        } else {
            $('#SMARTBILL_ORDER_STATUS').parent().parent().show();
        }
        $j('#SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT_off').on('change', function (e) {
            $('#SMARTBILL_ORDER_STATUS').parent().parent().hide();
        });
        $j('#SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT_on').on('change', function (e) {
            $('#SMARTBILL_ORDER_STATUS').parent().parent().show();
        });
    }

    function _initMoveInvoicePannelBackOffice() {
        //1.7.6
        if ($("#total_order").length) {
            $j('#smartbill-invoice-panel-backoffice').prependTo(".col-lg-5:last");
        }
        //1.7.7
        if ($("#update_order_status_action_form").length) {
            $j('#smartbill-invoice-panel-backoffice').prependTo("#order-view-page .col-md-8");
        }
    }

    function _initSyncStock() {
        if ($j('#SMARTBILL_SYNC_STOCK_off').prop('checked')) {
            $j('#SMARTBILL_USED_STOCK').prop("disabled", true);
            $j('input[name=SMARTBILL_SAVE_STOCK_HISTORY]').prop("disabled", true);
            $j('#smartbill-download-sync-stock-history').hide();
            $j('.token_details').addClass('greyed_out_text');
            $j('#smartbill-manually-sync-stock').hide();
        } else {
            $j('#SMARTBILL_USED_STOCK').prop("disabled", false);
            $j('input[name=SMARTBILL_SAVE_STOCK_HISTORY]').prop("disabled", false);
            $j('#smartbill-manually-sync-stock').show();
            if ($j('#SMARTBILL_SAVE_STOCK_HISTORY_off').prop('checked')) {
                $j('#smartbill-download-sync-stock-history').hide();
            } else {
                $j('#smartbill-download-sync-stock-history').show();
            }
        }
        $j('#SMARTBILL_SYNC_STOCK_off').on('change', function (e) {
            $j('#smartbill-manually-sync-stock').hide();
            $j('#SMARTBILL_USED_STOCK').prop("disabled", true);
            $j('#smartbill-download-sync-stock-history').hide();
            $j('input[name=SMARTBILL_SAVE_STOCK_HISTORY]').prop("disabled", true);
            $j('.token_details').addClass('greyed_out_text');
        });
        $j('#SMARTBILL_SYNC_STOCK_on').on('change', function (e) {
            $j('#smartbill-manually-sync-stock').show();
            $j('#SMARTBILL_USED_STOCK').prop("disabled", false);
            $j('input[name=SMARTBILL_SAVE_STOCK_HISTORY]').prop("disabled", false);
            $j('.token_details').removeClass('greyed_out_text');
            if ($j('#SMARTBILL_SAVE_STOCK_HISTORY_off').prop('checked')) {
                $j('#smartbill-download-sync-stock-history').hide();
            } else {
                $j('#smartbill-download-sync-stock-history').show();
            }
        });

        if ($j('#SMARTBILL_SAVE_STOCK_HISTORY_off').prop('checked')) {
            $j('#smartbill-download-sync-stock-history').hide();
        } else {
            $j('#smartbill-download-sync-stock-history').show();
        }
        $j('#SMARTBILL_SAVE_STOCK_HISTORY_off').on('change', function (e) {
            $j('#smartbill-download-sync-stock-history').hide();
        });
        $j('#SMARTBILL_SAVE_STOCK_HISTORY_on').on('change', function (e) {
            $j('#smartbill-download-sync-stock-history').show();
        });
    }

    function _initIssueWithDueDate() {
        if ($j('#SMARTBILL_ISSUE_WITH_DUE_DATE_off').prop('checked')) {
            $('#SMARTBILL_DUE_DAYS').prop("disabled", true);
        }
        else {
            $('#SMARTBILL_DUE_DAYS').prop("disabled", false);
        }

        $j('#SMARTBILL_ISSUE_WITH_DUE_DATE_off').on('change', function (e) {
            $('#SMARTBILL_DUE_DAYS').prop("disabled", true);
        });
        $j('#SMARTBILL_ISSUE_WITH_DUE_DATE_on').on('change', function (e) {
            $('#SMARTBILL_DUE_DAYS').prop("disabled", false);
        });
    }

    function _initIssueWithDeliveryDays() {
        if ($j('#SMARTBILL_ISSUE_WITH_DELIVERY_DAYS_off').prop('checked')) {
            $('#SMARTBILL_DELIVERY_DAYS').prop("disabled", true);
        }
        else {
            $('#SMARTBILL_DELIVERY_DAYS').prop("disabled", false);
        }
        $j('#SMARTBILL_ISSUE_WITH_DELIVERY_DAYS_off').on('change', function (e) {
            $('#SMARTBILL_DELIVERY_DAYS').prop("disabled", true);
        });
        $j('#SMARTBILL_ISSUE_WITH_DELIVERY_DAYS_on').on('change', function (e) {
            $('#SMARTBILL_DELIVERY_DAYS').prop("disabled", false);
        });
    }

    function _initDisplayTransport() {
        if ($j('#SMARTBILL_INCLUDE_SHIPPING_off').prop('checked')) {
            $('#SMARTBILL_SHIPPING_NAME').prop("disabled", true);
            $('#SMARTBILL_FREE_SHIPPING').prop("disabled", true);
        }
        else {
            $('#SMARTBILL_SHIPPING_NAME').prop("disabled", false);
            $('#SMARTBILL_FREE_SHIPPING').prop("disabled", false);
        }
        $j('#SMARTBILL_INCLUDE_SHIPPING_off').on('change', function (e) {
            $('#SMARTBILL_SHIPPING_NAME').prop("disabled", true);
            $('#SMARTBILL_FREE_SHIPPING').prop("disabled", true);
        });
        $j('#SMARTBILL_INCLUDE_SHIPPING_on').on('change', function (e) {
            $('#SMARTBILL_SHIPPING_NAME').prop("disabled", false);
            $('#SMARTBILL_FREE_SHIPPING').prop("disabled", false);
        });
    }


    function _initIssueWithDelegateData() {
        if ($j('#SMARTBILL_ADD_DELEGATE_DATA_off').prop('checked')) {
            $('#SMARTBILL_ISSUER_NAME').prop("disabled", true);
            $('#SMARTBILL_ISSUER_CNP').prop("disabled", true);
            $('#SMARTBILL_DELEGATE_NAME').prop("disabled", true);
            $('#SMARTBILL_DELEGATE_BULLETIN').prop("disabled", true);
            $('#SMARTBILL_DELEGATE_AUTO').prop("disabled", true);
        }
        else {
            $('#SMARTBILL_ISSUER_NAME').prop("disabled", false);
            $('#SMARTBILL_ISSUER_CNP').prop("disabled", false);
            $('#SMARTBILL_DELEGATE_NAME').prop("disabled", false);
            $('#SMARTBILL_DELEGATE_BULLETIN').prop("disabled", false);
            $('#SMARTBILL_DELEGATE_AUTO').prop("disabled", false);
        }
        $j('#SMARTBILL_ADD_DELEGATE_DATA_off').on('change', function (e) {
            $('#SMARTBILL_ISSUER_NAME').prop("disabled", true);
            $('#SMARTBILL_ISSUER_CNP').prop("disabled", true);
            $('#SMARTBILL_DELEGATE_NAME').prop("disabled", true);
            $('#SMARTBILL_DELEGATE_BULLETIN').prop("disabled", true);
            $('#SMARTBILL_DELEGATE_AUTO').prop("disabled", true);
        });
        $j('#SMARTBILL_ADD_DELEGATE_DATA_on').on('change', function (e) {
            $('#SMARTBILL_ISSUER_NAME').prop("disabled", false);
            $('#SMARTBILL_ISSUER_CNP').prop("disabled", false);
            $('#SMARTBILL_DELEGATE_NAME').prop("disabled", false);
            $('#SMARTBILL_DELEGATE_BULLETIN').prop("disabled", false);
            $('#SMARTBILL_DELEGATE_AUTO').prop("disabled", false);
        });
    }

    // Transform issue document links to ajax requests
    function _initAjaxCalls() {
        if ($('.smrt-ajax').length) {
            $('.smrt-ajax').each(function (i) {
                $(this).on('click', async function (e) {
                    if (!e.detail || e.detail == 1) {
                        e.preventDefault();
                        // show loading gif
                        $('#ajax_running').show();
                        $(this).addClass('disabled');
                        $('.smrt-doc').hide();

                        var api_call = $(this).data('href');

                        if ($(this).hasClass('reissue')) {
                            var attention_text = document.createElement("div");
                            attention_text.innerHTML = "Doriti reemiterea documentului in SmartBill Cloud?<br/>Va recomandam sa anulati sau sa stergeti documentul emis anterior in SmartBill Cloud inainte de reemitere.";
                            swal.fire({
                                title: "Atentie!",
                                html: attention_text,
                                icon: 'warning',
                                buttonsStyling: true,
                                reverseButtons: true,
                                showCancelButton: true
                            }).then(async function (result) {
                                if (result.isDismissed) {
                                    $('.smrt-ajax').removeClass('disabled');
                                    $('.smrt-doc').show();
                                    return false;
                                }
                                await call_issue_doc(api_call).then(function () {
                                    displayMessages();
                                });
                            });
                        } else {
                            await call_issue_doc(api_call).then(function () {
                                displayMessages();
                            });
                        }
                    } else {
                        return false;
                    }
                })
            });
        }
        if ($('.smrt-ajax-mail').length) {
            $('.smrt-ajax-mail').each(function () {
                $(this).on('click', async function (e) {
                    if (!e.detail || e.detail == 1) {
                        e.preventDefault();
                        var api_call = $(this).attr('data-href');
                        await call_send_doc(api_call).then(function () {
                            displayMessages();
                        });
                    } else {
                        return false;
                    }
                });
            });
        }

        $j('#smartbill-download-sync-stock-history').on('click', function (e) {
            e.preventDefault();
            var stock_url = ajax_url + "&action=downloadStockHistory";
            $.ajax({
                url: stock_url,
                xhr: function () {
                    var xhr = new XMLHttpRequest();
                    xhr.onreadystatechange = function () {
                        if (xhr.readyState == 2) {
                            if (xhr.status == 200) {
                                xhr.responseType = "blob";
                            } else {
                                xhr.responseType = "text";
                            }
                        }
                    };
                    return xhr;
                },
                success: function (data) {
                    var blob = new Blob([data], { type: "application/octetstream" });

                    //Check the Browser type and download the File.
                    var isIE = false || !!document.documentMode;
                    if (isIE) {
                        window.navigator.msSaveBlob(blob, "smartbill_sincronizare_stocuri.zip");
                    } else {
                        var url = window.URL || window.webkitURL;
                        link = url.createObjectURL(blob);
                        var a = $("<a />");
                        a.attr("download", "smartbill_sincronizare_stocuri.zip");
                        a.attr("href", link);
                        $("body").append(a);
                        a[0].click();
                        $("body").remove(a);
                    }
                },
                error: function (error) {
                    console.log(error)
                }
            });

        });
    }

    function _addOrderColumn() {
        //prestashop 1.7.7
        let table_order2 = '#order_grid_table';
        if ($(table_order2).length) {
            var table_order_head = $('#order_grid_table thead tr');
            var smrt_head = document.createElement('th');
            smrt_head.innerHTML = 'SmartBill';
            $(table_order_head)[0].append(smrt_head);

            var table_order_body = $('#order_grid_table tbody tr');
            table_order_body.each(function () {
                var order_id = $(this).find('input[type="checkbox"]').val();
                var smrt_col = document.createElement('td');
                smrt_col.className = 'text-right smartbill_prestashop_invoice';
                if (typeof smrt_docs != 'undefined' && typeof smrt_docs[order_id] != 'undefined') {
                    smrt_col.innerHTML = '<a style="padding:3px;font-size: 13px;line-height:1;" href="' + smrt_docs[order_id]['smartbill_document'] + '" target="_blank" class="btn btn-outline-secondary">' + smrt_docs[order_id]['series'] + ' ' + smrt_docs[order_id]['number'] + '</a>';
                }
                $(this).append(smrt_col);
            });
        }

        let table_order = '#table-order';
        if (!$(table_order).length) {
            table_order = '.table.order';
        }
        if ($(table_order).length) {
            $(table_order + ' thead tr').each(function (j) {
                var smrt_head = document.createElement('th');
                smrt_head.innerHTML = '';
                if (!j) {
                    smrt_head.innerHTML = 'SmartBill';
                }
                $(this).append(smrt_head);
            });
            $(table_order + ' tbody tr').each(function () {
                // smrt_docs is generated in hookDisplayBackOfficeHeader
                var order_id = $(this).find('input[type="checkbox"]').val();
                var smrt_col = document.createElement('td');
                smrt_col.className = 'text-right smartbill_prestashop_invoice';
                if (typeof smrt_docs != 'undefined' && typeof smrt_docs[order_id] != 'undefined') {
                    smrt_col.innerHTML = '<a href="' + smrt_docs[order_id]['smartbill_document'] + '" target="_blank" class="btn btn-default"><i class="icon-file" style="padding-right:4px"></i>' + smrt_docs[order_id]['series'] + ' ' + smrt_docs[order_id]['number'] + '</a>';
                }
                $(this).append(smrt_col);
            });
        }
    }

    async function call_issue_doc(api_call, show_toast = true) {
        if (show_toast) {
            var info_text = document.createElement("div");
            info_text.innerHTML = "Va rugam asteptati raspunsul serverului...";
            swal.fire({
                title: 'Se incarca!',
                html: info_text,
                icon: 'info',
                buttonsStyling: true,
                reverseButtons: true,
                showConfirmButton: false
            });
        }
        let result = await $.ajax({
            url: api_call,
            success: function (data) {
                // hide loading gif
                $('#ajax_running').hide();
                $('.smrt-ajax').removeClass('disabled');
                data = JSON.parse(data);
                if (show_toast == true) {
                    swal.close();
                }
                if (typeof data.error !== 'undefined' && !data.status) {
                    if (data.error == '') {
                        data.error = 'Verificati setarile modulului.';
                    }
                    if (data.error === "408") {
                        data.error = "A aparut o eroare la contactarea serverului SmartBill.Acceseaza cloud.smartbill.com, anuleaza factura pentru comanda #" + data.orderId + " si incearca din nou facturarea comenzii din magazinul online.";
                    } else if (typeof data.orderId !== 'undefined') {
                        data.error = "Factura pentru comanda #" + data.orderId + " nu a fost emisa in SmartBill. <br>" + data.error;
                    }
                    errorMessages.push(data.error);
                } else {
                    action = "emis";
                    if (!$('a.smrt-doc').length) {
                        var smartbill_link = document.createElement('a');
                        smartbill_link.href = data.documentUrl;
                        smartbill_link.target = "_blank";
                        smartbill_link.className = "btn btn-primary smrt-doc";
                        smartbill_link.innerHTML = '<i class="icon-file"></i> Vizualizeaza in SmartBill';
                        smartbill_link.style.color = 'white!important';
                        $('.smartbill-container').append(smartbill_link);

                        var smartbill_link2 = document.createElement('a');
                        smartbill_link2.className = "btn btn-primary smrt-ajax-mail";
                        smartbill_link2.dataset.href = ajax_url + smartbill.post_id + "&action=sendMailDocument";
                        smartbill_link2.innerHTML = '<i class="icon-file"></i> Trimite documentul clientului';
                        smartbill_link2.style.color = 'white';
                        smartbill_link2.style.marginLeft = '3px';
                        $('.smartbill-container').append(smartbill_link2);

                        $(smartbill_link2).click(async function (e) {
                            if (!e.detail || e.detail == 1) {
                                e.preventDefault();
                                var api_call = $(this).attr('data-href');
                                await call_send_doc(api_call).then(function () {
                                    displayMessages();
                                });
                            } else {
                                return false;
                            }
                        });

                    } else {
                        action = "reemis";
                        $('a.smrt-doc').attr('href', data.documentUrl).show();
                    }

                    $('.smartbill-container a.smrt-ajax').addClass('reissue').html('<i class="icon-exchange"></i> Reemite document');

                    if (successMessages != 0) {
                        successMessages = ['Documentele au fost emise cu succes.'];
                    }
                    else {
                        successMessages.push("Documentul " + data.series + ' ' + data.number + " a fost " + action + " cu succes.");
                    }
                }
                saveMessages();
                return true;
            }
        });
        return result;
    }

    async function call_send_doc(api_call, show_toast = true) {
        if (show_toast) {
            var info_text = document.createElement("div");
            info_text.innerHTML = "Va rugam asteptati raspunsul serverului...";
            swal.fire({
                title: 'Se incarca!',
                html: info_text,
                icon: 'info',
                buttonsStyling: true,
                reverseButtons: true,
                showConfirmButton: false
            });
        }
        let result = await $.ajax({
            url: api_call,
            success: function (data) {
                data = JSON.parse(data);

                if (show_toast == true) {
                    swal.close();
                }
                if (typeof data.error !== 'undefined' && !data.status) {
                    if (data.error == '') {
                        data.error = 'Verificati setarile modulului.';
                    }
                    if (data.error === "408") {
                        data.error = "A aparut o eroare la contactarea serverului SmartBill. Acceseaza cloud.smartbill.com, anuleaza factura pentru comanda #" + data.orderId + " si incearca din nou facturarea comenzii din magazinul online.";
                    }
                    errorMessages.push(data.error);
                } else {
                    if (successMessages != 0) {
                        successMessages = ['Documentele au fost emise cu succes.'];
                    }
                    else {
                        successMessages.push(data.message);
                    }
                }
                saveMessages();
                return true;
            }
        });
        return result;
    }

    async function asyncForEach(array, callback) {
        for (let index = 0; index < array.length; index++) {
            await callback(array[index], index, array);
        }
    }
    const waitFor = (ms) => new Promise(r => setTimeout(r, ms));

    var successMessages = [];
    var errorMessages = [];

    function displayMessages() {
        var path = window.location.pathname;
        if (typeof (Storage) !== "undefined") {
            if (localStorage.path != 'undefined') {
                if (path == localStorage.path) {
                    if (localStorage.successMessages != '' || localStorage.successMessages != 'undefined') {
                        var storedMessages = JSON.parse(localStorage.successMessages);
                        storedMessages.forEach(function (item, index, array) {
                            Toastify({
                                text: item, duration: 3000, newWindow: false, close: true, gravity: "top", position: 'center', backgroundColor: "#00A14B", stopOnFocus: true,
                            }).showToast();
                            if (index === array.length - 1) {
                                localStorage.removeItem("successMessages");
                            }
                        });
                        successMessages = [];
                    }
                    if (localStorage.errorMessages != '' || localStorage.errorMessages != 'undefined') {
                        var storedMessages = JSON.parse(localStorage.errorMessages);
                        storedMessages.forEach(function (item, index, array) {
                            Toastify({
                                text: item, duration: -1, newWindow: false, close: true, gravity: "top", position: 'center', backgroundColor: "#EF4136", stopOnFocus: true,
                            }).showToast();
                            if (index === array.length - 1) {
                                localStorage.removeItem("errorMessages");
                            }
                        });
                        errorMessages = [];
                    };

                    localStorage.removeItem('path');
                } else {
                    localStorage.removeItem('path');
                    localStorage.removeItem("successMessages");
                    localStorage.removeItem("errorMessages");
                };
            };
        };
    }
    function saveMessages() {
        if (typeof (Storage) !== "undefined") {
            if (localStorage.successMessages != 'undefined') {
                localStorage.successMessages = JSON.stringify(successMessages);
            }
            if (localStorage.errorMessages != 'undefined') {
                localStorage.errorMessages = JSON.stringify(errorMessages);
            }
            if (localStorage.path != 'undefined') {
                localStorage.path = window.location.pathname;
            }
        }
    }

    // public methods
    return {
        init: _init
    }
})(jQuery);

// init on ready
$(document).ready(function () {
    SmartBillCloud.init();
});
